/*
    SETUP.PRG
    A setup program for Ken's Sample InterBase application

    This program can be run by itself, or it can be run
    via the START program ...
*/ 

/*
   First things first, see if the BDE Alias exists, and if 
   not, let's create it.

   We will use the BDEALIAS.CC from the DUFLP library ...
*/

set procedure to message.wfm additive // message form
set procedure to bdealias.cc additive // bde alias control file
b = new BDEAlias()

fMessage = new MessageForm()
fMessage.text         := "Setup for Ken's InterBase Sample"
fMessage.title.text   := "Setup"
fMessage.message.text := "Checking for BDE Alias"
fMessage.Open()

// see if the alias exists:
cBDEAliasName = "KENIBSAMPLE"
cDataBaseFile = "KENSAMPLE.GDB"
if not b.databaseFound( cBDEAliasName )
   // not found, so we need to create it, but only
   // if the file KENIBSAMPLE.GDB exists in the 
   // current directory:
   cPath = set("DIRE")
   cDatabase = cPath+"\"+cDatabaseFile

   if not file( cDatabase )
      msgbox( "Database "+cDatabaseFile+" does not exist in the "+;
              "working directory for the application!", ;
              "Can't do it!", 16 )
      fMessage.close()
      fMessage.release()
      return
   endif

   // update message form ...
   fMessage.message.text := "Creating BDE Alias"
   fMessage.repaint()

   // ok, it exists, create the alias for it ...
   cIPath = cPath+";"+;
           "SERVER NAME:"+cDatabase+";"+;
           "USER NAME:SYSDBA;"+;
           "SQLQRYMODE:SERVER;"+;
           "SQLPASSTHRU MODE:SHARED NOAUTOCOMMIT"

   if not b.createAlias( cBDEAliasName, cIPath, "INTRBASE" )
      msgbox( "Could not create BDE Alias "+cBDEAliasName+;
              " for some reason. Check to see if everything "+;
              "is set up correctly ...", "Could not do it!", 16 )
      fMessage.close()
      fMessage.release()
      close procedure fMessage
      return
   endif

   // release the BDE Alias object, as it is no longer
   // needed:
   release b

endif

// create database object ...:
d = new Database()
d.databaseName := cBDEAliasName
d.share       := 1
d.loginString  := "SYSDBA/masterkey"
try
   d.active       := true
catch (dbException e)
   if e.Code == 240 // probably InterBase Server not running
      msgbox( "InterBase Server may not be running. Otherwise, "+;
              "some unknown error has occurred.", "Can't do it!", 16 )
   endif
   d.active := false
   release d
   fMessage.close()
   fMessage.release()
   cancel
endtry

// update message form ...
fMessage.message.text := "Checking for Sample Table"
fMessage.repaint()

// check for the table:
if not d.tableExists( "FISH" )
   fMessage.message.text := "Creating Sample Table"
   fMessage.repaint()

   // we're going to copy the FISH table from the
   // VDBSample database (first make sure we can catch an
   // error if the VDBSAMPLE database doesn't exist):
   try
      dSample = new Database()
      dSample.databaseName := "VDBSAMPLE"
      dSample.active       := true
   catch (Exception e)
      msgbox( "Could not open 'VDBSAMPLE' database -- "+;
              "check the BDE Administrator to see if it exists ...",;
              "Could not do it!", 16 )
      dSample.active := false // just to be safe
      release dSample
      d.Active := false
      release d
      return
   endtry

   // first we need to create the fish table:
   // For this to work, the only way I've been able to 
   // create the table is via LOCAL Sql -- not the passthrough
   // method of the database class. This means deactivating
   // the database object, opening the database, and so on ...
   // when done, of course, you have to close the database,
   // and then re-activate the database object ...
   d.active := false
   open database kenibsample
   set database to kenibsample
   CREATE TABLE fish ( ID INTEGER , ;
                       Name VARCHAR( 30 ) NOT NULL PRIMARY KEY, ;
                       Species VARCHAR( 40 ), ;
                       Length_CM NUMERIC(8,2),  ;
                       Description BLOB(10,1),  ;
                       OLE_Graphic BLOB(10,4), ;
                       NameShadow  VARCHAR(30), ;
                       TestLogical VARCHAR(1) DEFAULT "N" )

   // create a indexes:
   CREATE INDEX name ON fish (NameShadow)
   CREATE INDEX id ON fish (id) //DISTINCT // unique
   CREATE INDEX species ON fish (species)

   close database kenibsample
   d.active := true

   // now to use UpdateSet:
   qFish = new Query()
   qFish.database := dSample
   qFish.sql      := "select * from fish"
   qFish.active   := true

   u = new UpdateSet()
   u.Source      := qFish.rowset
   u.Destination := ":"+cBDEAliasName+":FISH"
   u.Append()

   // that should be it except that copy does not
   // copy the indexes ... and we want to add a logical
   // field to this table for purposes of showing
   // how to morph a checkbox's value ...
   dSample.active := false
   release qFish, u, dSample

   // the ID field should now be either INTEGER or SMALLINT
   // in type, and will need to be set to use a generator
   // and a trigger to ensure that it autoincrements. We
   // will also want to find the last value so we can
   // set the generator to that value -- otherwise the
   // generator will start at 1 again ...
   qFish = new Query()
   qFish.database := d
   qFish.sql      := "select * from fish"
   qFish.active   := true
   qFish.rowset.indexName := "ID"
   qFish.rowset.last()
   nGen = qFish.rowset.fields["ID"].value
   qFish.active       := false
   release qFish
   fMessage.message.text := "Creating Generator/Trigger"
   fMessage.repaint()

   // create the generator:
   cCreateGen = "CREATE GENERATOR next_ID"
   try
      d.executeSQL( cCreateGen )
   catch (dbException e)
      msgbox( "Could not create generator -- it might already exist.",;
              "Could not do this ...", 16 )
   endtry

   // set the generator value:
   cSetGen    = "SET GENERATOR next_ID to "+nGen
   d.executeSQL( cSetGen )

   // create the trigger -- this is what
   // causes the "autoincrement" to occur:
   cCreateTrigger = "CREATE TRIGGER createID FOR fish "+;
                    "ACTIVE BEFORE INSERT POSITION 0 "+;
                    "AS BEGIN new.ID = GEN_ID( next_ID, 1 ); END"
   try
      d.executeSQL( cCreateTrigger )
   catch( dBException e )
      msgbox( "Could not create trigger createID -- it might already "+;
              "exist.", "Could not do this ...", 16 )
   endtry

   fMessage.message.text := "Putting Data Into Shadow Field "+;
                            "and Creating Triggers"
   fMessage.repaint()

   // need to make sure data is copied to NameShadow field
   // for rows that already exist:
   cUpdate = "UPDATE fish SET NameShadow = UPPER( name )"
   d.executeSQL( cUpdate )
   
   // Now for the triggers:
   cTrigger1 = "CREATE TRIGGER nsAppend FOR fish "+;
               "BEFORE INSERT "+;
               "AS BEGIN new.NameShadow = UPPER( new.Name ); END"
   d.executeSQL( cTrigger1 )
   cTrigger2 = "CREATE TRIGGER nsUpdate FOR fish "+;
               "BEFORE UPDATE "+;
               "AS BEGIN new.NameShadow = UPPER( new.Name ); END"
   d.executeSQL( cTrigger2 )

   fMessage.message.text := "Creating Indexes"
   fMessage.repaint()
   // ok, we have triggers, we have a generator, there's
   // nothing left to do, I think ...

endif


fMessage.message.text := "Starting application"
fMessage.repaint()

// insert whatever else is necessary here ...
close procedure

set procedure to genericSDIApp.cc additive
set procedure to sample.mnu       additive
set procedure to custbutt.cc      additive
set procedure to formcntl.cc      additive
// others


fMessage.close()
fMessage.release()

// -- end of SETUP program

